// Per a programar la botonera de cabina cal que A0 sigui a massa (jumper posat) !!!!!

/*
LIFT CABIN BUTTONS
BOTONERA CABINA

www.jorda-sa.es

Controlling software by Xavier Pi
Hardware and firmware by Jordi Binefa

20230612

 * 3V3 i 3.3V són punts elèctrics diferents!
 * SDA i SCL estan intercanviats a la primera versió
 * 
The ESP32 chip has the following strapping pins:
GPIO 0 (must be LOW to enter boot mode)
GPIO 2 (must be floating or LOW during boot)
GPIO 4.
GPIO 5 (must be HIGH during boot)
GPIO 12 (must be LOW during boot)
GPIO 15 (must be HIGH during boot)

* Register:
b7 b6 b5 b4 b3 b2 b1 b0
x  x  x  x  0  0  0  1     REG_RD
x  x  x  x  0  0  1  0     REG_WR
0  0  0  0  0  1  1  0     LCB_VERS

** Register REG_RD:
b7 b6 b5 b4 b3 b2 b1 b0
x  x  x  x  0  0  0  1     REG_RD

b7: 0 <-- Reading byte
b7: 1 <-- Reading bit

*** Reading byte:
Raspberry Pi needs two I2C sendings to read properly.

pi@raspberrypi:~ $ i2cset -y 1 0x37 0x02 0x00
pi@raspberrypi:~ $ i2cget -y 1 0x37 0x01
0x3f
pi@raspberrypi:~ $ i2cget -y 1 0x37 0x01
0x00
pi@raspberrypi:~ $ i2cset -y 1 0x37 0x02 0x3f
pi@raspberrypi:~ $ i2cget -y 1 0x37 0x01
0x00
pi@raspberrypi:~ $ i2cget -y 1 0x37 0x01
0x3f

*** Reading bit:
Raspberry Pi needs two I2C sendings to read properly.

Sending 1 to button led 6 (It is mapped as 5):
i2cset -y 1 0x37 0x02 0xd1

Sending 1 to button led 1 (It is mapped as 0):
i2cset -y 1 0x37 0x02 0x81

Reading button's led 6 (It is mapped as 5):
pi@raspberrypi:~ $ i2cget -y 1 0x37 0xd1
0x30
pi@raspberrypi:~ $ i2cget -y 1 0x37 0xd1
0x01

Sending 0 to button led 6 (It is mapped as 5):
pi@raspberrypi:~ $ i2cset -y 1 0x37 0x02 0xd0

Reading button's led 6 (It is mapped as 5):
pi@raspberrypi:~ $ i2cget -y 1 0x37 0xd1
0x01
pi@raspberrypi:~ $ i2cget -y 1 0x37 0xd1
0x00

Reading button's led 1 (It is mapped as 0):
pi@raspberrypi:~ $ i2cget -y 1 0x37 0x81
0x00
pi@raspberrypi:~ $ i2cget -y 1 0x37 0x81
0x01

Sending 0 to button led 1 (It is mapped as 0):
pi@raspberrypi:~ $ i2cset -y 1 0x37 0x02 0x80

Reading button's led 1 (It is mapped as 0):
pi@raspberrypi:~ $ i2cget -y 1 0x37 0x81
0x01
pi@raspberrypi:~ $ i2cget -y 1 0x37 0x81
0x00

** Register REG_WR:
b7 b6 b5 b4 b3 b2 b1 b0
x  x  x  x  0  0  1  0     REG_WR

b7: 0 <-- Sending byte
b7: 1 <-- Sending bit

*** Sending byte:
6 leds ON:
i2cset -y 1 0x37 0x02 0x3f

6 leds OFF:
i2cset -y 1 0x37 0x02 0x00

Only led 1 ON:
i2cset -y 1 0x37 0x02 0x01

*** Sending bit:
Sending 1 to button led 1
i2cset -y 1 0x37 0x02 0x81

Sending 0 to button led 1
i2cset -y 1 0x37 0x02 0x80

Sending 1 to button led 2
i2cset -y 1 0x37 0x02 0x91

Sending 0 to button led 2
i2cset -y 1 0x37 0x02 0x90

Sending 1 to button led 3
i2cset -y 1 0x37 0x02 0xa1

Sending 0 to button led 3
i2cset -y 1 0x37 0x02 0xa0

Sending 1 to button led 4
i2cset -y 1 0x37 0x02 0xb1

Sending 0 to button led 4
i2cset -y 1 0x37 0x02 0xb0

Sending 1 to button led 5
i2cset -y 1 0x37 0x02 0xc1

Sending 0 to button led 5
i2cset -y 1 0x37 0x02 0xc0

Sending 1 to button led 6
i2cset -y 1 0x37 0x02 0xd1

Sending 0 to button led 6
i2cset -y 1 0x37 0x02 0xd0

** Register VERSION:
b7 b6 b5 b4 b3 b2 b1 b0
x  x  x  x  x  1  1  0     VERT_VERS

Raspberry Pi needs two I2C sendings to read properly.
i2cget -y 1 0x37 0x06 w
0x2030
i2cget -y 1 0x37 0x06 w
0x0100

Answer 0x0100:
01: VERSION
00: SUBVERSION

*/
#include <Wire.h>

#define I2C_ADDR_BASE 0x30

#define LCB_VERSION    0x01
#define LCB_SUBVERSION 0x00

#define LED_1 4 /* Strapping pin */
#define LED_2 5 /* Strapping pin */
#define LED_3 12 /* Strapping pin */
#define LED_4 13
#define LED_5 14
#define LED_6 15 /* Strapping pin */
#define LED_BRD 32
#define A0 2 /* Strapping pin !!!! */
#define A1 18 /* A la versió 00 punt elèctric compartit amb sortida de 24V */
#define A2 19 /* A la versió 00 punt elèctric compartit amb sortida de 24V */
#define BT_IO0 0 /* Strapping pin */
#define WEB 23
#define BT1 25
#define BT2 26
#define BT3 27
#define BT4 33
#define BT5 34
#define BT6 35
#define SDA 22 /* v2.1 Reparat --> La connexió al bus és a l'inrevés. SDA <--> SCL*/
#define SCL 21 /* v2.1 Reparat --> La connexió al bus és a l'inrevés. SDA <--> SCL*/
#define RX2 16
#define TX2 17

#define bLed1Mask 0x01
#define bLed2Mask 0x02
#define bLed3Mask 0x04
#define bLed4Mask 0x08
#define bLed5Mask 0x10
#define bLed6Mask 0x20
#define bLedBrdMask 0x40

#define REG_RD    0x01
#define REG_WR    0x02
#define LCB_VERS  0x06

TwoWire I2C_M = TwoWire(0); // Main I2C. ESP32-S3 acting as slave
byte byLastReadNumber,byLeds;

byte byReadI2cAddress(){
  byte byAddr = (digitalRead(A2))?0x00:0x4;
  byAddr |= (digitalRead(A1))?0x00:0x2;
  byAddr |= (digitalRead(A0))?0x00:0x1;
  
  return I2C_ADDR_BASE | byAddr;
}

void onRequest(){ 
  word wAnswer;

  Serial.println("onRequest");
  Serial.print("byLastReadNumber: 0x");
  Serial.println(byLastReadNumber,HEX);

  if((byLastReadNumber & 0x0F) == REG_RD){
    if(!(byLastReadNumber & 0x80)){ // b7: 0 - Byte level
      I2C_M.write(byLeds);
      Serial.print("Esclau respon 0x");
      Serial.print(byLeds,HEX);
    }else{ // b7: 1 - Bit level
      I2C_M.write((byLeds & (byte)(1 << (0x07 & (byLastReadNumber>>4))))?0x01:0x00);
      Serial.print("Esclau respon 0x");
      Serial.print((byLeds & (byte)(1 << (0x07 & (byLastReadNumber>>4))))?0x01:0x00,HEX);
    }
  }
  if((byLastReadNumber & 0x0F) == LCB_VERS){
    I2C_M.write(LCB_SUBVERSION);
    I2C_M.write(LCB_VERSION);
    Serial.print("Esclau respon 0x");
    Serial.print(LCB_SUBVERSION,HEX);
    Serial.print(" 0x");
    Serial.println(LCB_VERSION,HEX);
  }
}

void onReceive(int len){
  enum {rd,wr,versio,no} enReg = no;
  byte byReg,byWhichDev = 0x00;
  int n = 0;
  short int siN;
  
  Serial.printf("onReceive[%d]: ", len);

  digitalWrite(LED_BRD,HIGH);
  
  while (1 < I2C_M.available()) { // loop through all but the last
    byLastReadNumber = I2C_M.read();   // receive byte as a character
    Serial.print(n,DEC);
    Serial.print(") El màster m'ha fet arribar un 0x");
    Serial.println(byLastReadNumber,HEX);
    switch(n){
      case 0:
        Serial.print("byLastReadNumber & 0x07: 0x");
        Serial.println(byLastReadNumber & 0x07,HEX);
        switch(byLastReadNumber & 0x07){
          case REG_RD:  enReg = rd; Serial.println("enReg = rd");break;
          case REG_WR:  enReg = wr; Serial.println("enReg = wr");    break;
          case LCB_VERS: enReg = versio; Serial.println("enReg = versio");break;
          default: enReg = no; Serial.println("enReg = no");
        }
        byReg = byLastReadNumber;
        break;
    }
    n++;
  }
  byLastReadNumber = I2C_M.read();
  Serial.print(n,DEC);
  Serial.print("> El màster m'ha fet arribar un 0x");
  Serial.println(byLastReadNumber,HEX);
  switch(n){
    case 1:
      switch(enReg){
        case wr:
          if(!(byLastReadNumber & 0x80)){ // b7: 0 - Byte level
            byLeds = byLastReadNumber;
            Serial.print("byLeds: 0x");
            Serial.println(byLeds,HEX);            
          }else{ // b7: 1 - Bit level
            if( byLastReadNumber & 0x01 ){
              byLeds |= (byte)(1 << (0x07 & (byLastReadNumber>>4)));
            }else{
              byLeds &= ~(byte)(1 << (0x07 & (byLastReadNumber>>4)));
            }
            Serial.print("Sending ");
            Serial.print( byLastReadNumber & 0x01 );
            Serial.print(" to button led ");
            Serial.print((0x07 & (byLastReadNumber>>4)) + 1);
            Serial.print(" - byLeds: 0x");
            Serial.println(byLeds,HEX);
          }
          break;
      }
      break;
  }

  digitalWrite(LED_BRD,LOW);
}

void setup() {
  byte byI2cAddr;
  uint32_t i = 0;
  
  Serial.begin(115200);
  Serial.println(__FILE__);
  pinMode(LED_1, OUTPUT);
  pinMode(LED_2, OUTPUT);
  pinMode(LED_3, OUTPUT);
  pinMode(LED_4, OUTPUT);
  pinMode(LED_5, OUTPUT);
  pinMode(LED_6, OUTPUT);
  pinMode(LED_BRD, OUTPUT);
  pinMode(A0, INPUT);
  pinMode(A1, INPUT);
  pinMode(A2, INPUT);
  pinMode(BT_IO0, INPUT);
  pinMode(WEB, INPUT_PULLUP);
  pinMode(BT1, INPUT);
  pinMode(BT2, INPUT);
  pinMode(BT3, INPUT);
  pinMode(BT4, INPUT);
  pinMode(BT5, INPUT);
  pinMode(BT6, INPUT);
  
  /*Serial.print("A0: ");
  (digitalRead(A0)) ? Serial.println("1") : Serial.println("0");
   Serial.print("A1: ");
  (digitalRead(A1)) ? Serial.println("1") : Serial.println("0");
   Serial.print("A2: ");
  (digitalRead(A2)) ? Serial.println("1") : Serial.println("0");*/

  byI2cAddr = byReadI2cAddress();
  Serial.print("I2C address: 0x");
  Serial.println(byI2cAddr,HEX);

  delay(5000); // Espero estabilització del bus I2C per part de la Raspberry

  I2C_M.onReceive(onReceive);
  I2C_M.onRequest(onRequest);
  I2C_M.begin((uint8_t)byI2cAddr,SCL,SDA,400000); /* La connexió al bus és a l'inrevés. SDA <--> SCL*/
  // I2C_M.begin((uint8_t)byI2cAddr,SDA,SCL,400000); 
  
#if CONFIG_IDF_TARGET_ESP32
  // https://docs.espressif.com/projects/arduino-esp32/en/latest/api/i2c.html
  char message[64];
  snprintf(message, 64, "%u Packets.", i++);
  I2C_M.slaveWrite((uint8_t *)message, strlen(message));
  Serial.println("CONFIG_IDF_TARGET_ESP32");
#endif

  byLeds = 0x00;
}

boolean bPressedButton(int nWhichOne) {
  if (digitalRead(nWhichOne))
    return false;
  return true;
}

void loop() {
  static boolean b1wasPressed=false,b2wasPressed=false,b3wasPressed=false;
  static boolean b4wasPressed=false,b5wasPressed=false,b6wasPressed=false;
  boolean b1currentState=bPressedButton(BT1),b2currentState=bPressedButton(BT2);
  boolean b3currentState=bPressedButton(BT3),b4currentState=bPressedButton(BT4);
  boolean b5currentState=bPressedButton(BT5),b6currentState=bPressedButton(BT6);

  if (b1wasPressed != b1currentState) {
    delay(2);
    if (b1currentState) {
      byLeds |= bLed1Mask;
      Serial.print("LED_1: ");
      (byLeds & bLed1Mask) ? Serial.println("ON") : Serial.println("OFF");
    }
    b1wasPressed = b1currentState;
  }
  if (b2wasPressed != b2currentState) {
    delay(2);
    if (b2currentState) {
      byLeds |= bLed2Mask;
      Serial.print("LED_2: ");
      (byLeds & bLed2Mask) ? Serial.println("ON") : Serial.println("OFF");
    }
    b2wasPressed = b2currentState;
  }
  if (b3wasPressed != b3currentState) {
    delay(2);
    if (b3currentState) {
      byLeds |= bLed3Mask;
      Serial.print("LED_3: ");
      (byLeds & bLed3Mask) ? Serial.println("ON") : Serial.println("OFF");
    }
    b3wasPressed = b3currentState;
  }   
  if (b4wasPressed != b4currentState) {
    delay(2);
    if (b4currentState) {
      byLeds |= bLed4Mask;
      Serial.print("LED_4: ");
      (byLeds & bLed4Mask) ? Serial.println("ON") : Serial.println("OFF");
    }
    b4wasPressed = b4currentState;
  }
  if (b5wasPressed != b5currentState) {
    delay(2);
    if (b5currentState) {
      byLeds |= bLed5Mask;
      Serial.print("LED_5: ");
      (byLeds & bLed5Mask) ? Serial.println("ON") : Serial.println("OFF");
    }
    b5wasPressed = b5currentState;
  }
  if (b6wasPressed != b6currentState) {
    delay(2);
    if (b6currentState) {
      byLeds |= bLed6Mask;
      Serial.print("LED_6: ");
      (byLeds & bLed6Mask) ? Serial.println("ON") : Serial.println("OFF");
    }
    b6wasPressed = b6currentState;
  }
  digitalWrite(LED_1,byLeds & bLed1Mask);
  digitalWrite(LED_2,byLeds & bLed2Mask);
  digitalWrite(LED_3,byLeds & bLed3Mask);
  digitalWrite(LED_4,byLeds & bLed4Mask);
  digitalWrite(LED_5,byLeds & bLed5Mask);
  digitalWrite(LED_6,byLeds & bLed6Mask);
}
